from . import Interfaces
from . import MediaPlayer2
from . import Player
from . import Playlists
from . import TrackList


if __name__ == '__main__':
    print(Interfaces())
    print(MediaPlayer2())
    print(Player())
    print(Playlists())
    print(TrackList())
